"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
# vim: set tw=80 ts=4 sw=4 sts=4:
import arrow

from functools import wraps

from flask import request, g
from flask.ext.restful import Resource, fields, reqparse, marshal_with, marshal_with_field

from sqlalchemy.orm.exc import NoResultFound

from werkzeug.exceptions import BadRequest

from accountant import app

from .. import api_api

from ..fields import Object

from ..models.users import User


def load_user_from_token(token):
    return User.verify_auth_token(token)


def load_user_from_auth(auth):
    token = auth.replace('Bearer ', '', 1)
    return load_user_from_token(token)


def load_user_from_request():
    # No token found, trying to authenticate using request data.
    try:
        kwargs = parser.parse_args()

        try:
            user = User.query().filter(
                User.email == kwargs['email']
            ).one()

            if user and user.verify_password(kwargs['password']):
                return user

        except NoResultFound:
            pass
    except BadRequest:
        pass

    return None


def authenticate():
    return {'error': 'Please login before executing this request.'}, 401


def requires_auth(f):
    @wraps(f)
    def wrapped(*args, **kwargs):
        if 'Authorization' in request.headers:
            auth = request.headers['Authorization']
            user = load_user_from_auth(auth)
        else:
            user = load_user_from_request()

        if user:
            g.user = user
            return f(*args, **kwargs)

        return authenticate()
    return wrapped


token_resource_fields = {
    'token': fields.String,
    'expiration': fields.DateTime(dt_format='iso8601'),
    'token_type': fields.String
}

user_resource_fields = {
    'id': fields.Integer(default=None),
    'email': fields.String,
    'active': fields.Boolean
}

parser = reqparse.RequestParser()
parser.add_argument('email', type=str, required=True)
parser.add_argument('password', type=str, required=True)


class LoginResource(Resource):
    @requires_auth
    @marshal_with(token_resource_fields)
    def post(self):
        user = g.user

        token = user.generate_auth_token()
        expiration_time = arrow.now().replace(
            seconds=app.config['SESSION_TTL']
        )

        return {
            'token': token,
            'expiration': expiration_time.datetime,
            'token_type': 'Bearer'
        }, 200

    @requires_auth
    @marshal_with_field(Object(user_resource_fields))
    def get(self):
        return g.user


api_api.add_resource(LoginResource, "/users/login")
